'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.evaluateModules = exports.ensureNoImports = undefined;

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.compileModulesFromSource = compileModulesFromSource;
exports.compileModule = compileModule;
exports.inspect = inspect;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _bluebird = require('bluebird');

var _ramda = require('ramda');

var _babelCore = require('babel-core');

var _compiler = require('./compiler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var readFile = (0, _bluebird.promisify)(_fs2.default.readFile);

/**
 * Ensures a piece of source has no import or require declarations
 *
 * @param {String} source
 * @return {Promise}
 */
var ensureNoImports = exports.ensureNoImports = (0, _ramda.curry)(function (filename, source) {
    var _inspect = inspect(source),
        modules = _inspect.modules;

    return modules.length ? (0, _bluebird.reject)(new Error('Cannot import modules on autocomplete files (' + filename + ')')) : (0, _bluebird.resolve)();
});

/**
 * Traverses the AST, getting the imported modules and compile to pairs
 *
 * @author Marcelo Haskel Camargo
 * @param {String} source
 * @return {Promise<String[][]>}
 */
function compileModulesFromSource(source) {
    var _inspect2 = inspect(source),
        modules = _inspect2.modules;

    return (0, _bluebird.all)(modules.filter((0, _ramda.startsWith)('./')).map(compileModule));
}

/**
 * Compiles a module to a pair with (filename :: string, source :: string)
 *
 * @author Marcelo Haskell Camargo
 * @param {String} module
 * @return {Promise}
 */
function compileModule(module) {
    var extensionIs = function extensionIs(extension) {
        return function () {
            return (0, _ramda.endsWith)(extension, module);
        };
    };

    return readFile(module, 'utf-8').then((0, _ramda.cond)([[extensionIs('.js'), _compiler.compileES6], [extensionIs('.json'), function () {
        return (0, _stringify2.default)(JSON.parse.apply(JSON, arguments));
    }], [_ramda.T, function () {
        return (0, _bluebird.reject)(new Error('Unknown module loader for file ' + module));
    }]])).then(function (source) {
        return [module, source];
    });
}

/**
 * Evaluates a list of pairs of modules. modules :: [(String, String)]
 *
 * @author Marcelo Haskell Camargo
 * @param {NodeVM} vm - Virtual machine instance to run
 * @param {String[][]} modules pairs, with (name :: string, source :: string)
 */
var evaluateModules = exports.evaluateModules = function evaluateModules(vm, modules) {
    return (0, _ramda.fromPairs)((0, _ramda.map)(function (_ref) {
        var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
            module = _ref2[0],
            source = _ref2[1];

        // JSON doesn't need to run on VM. We can directly parse it
        var convertToBytecode = (0, _ramda.cond)([[(0, _ramda.endsWith)('.json'), function () {
            return JSON.parse(source);
        }], [(0, _ramda.endsWith)('.js'), _2 => {
            return vm.run(source, _2);
        }], [_ramda.T, function (module) {
            throw new Error('Unknown file type for ' + module);
        }]]);

        return [module, convertToBytecode(module)];
    }, modules));
};

/**
 * Inspects a JS source and returns processed information, such as ES5 code,
 * the ast, source map and the used modules
 *
 * @author Marcelo Haskell Camargo
 * @param {String} source - ES6 source
 * @return {String[]}
 */
function inspect(source) {
    var modules = [];
    var result = (0, _babelCore.transform)(source, {
        comments: false,
        compact: true,
        presets: ['es2015', 'react'],
        plugins: [['transform-react-jsx', { pragma: '__render__' }], [function () {
            return {
                visitor: {
                    ImportDeclaration: function ImportDeclaration(_ref3) {
                        var node = _ref3.node;

                        modules.push(node.source.value);
                    },
                    CallExpression: function CallExpression(_ref4) {
                        var node = _ref4.node;

                        // Find and extract require(module)
                        var callee = (0, _ramda.path)(['callee', 'name'], node);

                        if (callee === 'require') {
                            var _node$arguments = (0, _slicedToArray3.default)(node.arguments, 1),
                                moduleNode = _node$arguments[0];

                            var isLiteralModule = moduleNode && moduleNode.type === 'StringLiteral';

                            if (isLiteralModule) {
                                modules.push(moduleNode.value);
                            }
                        }
                    }
                }
            };
        }]]
    });

    return (0, _ramda.merge)(result, { modules: (0, _ramda.uniq)(modules) });
}
//# sourceMappingURL=module.js.map