'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.runInBrowser = undefined;
exports.createVM = createVM;
exports.runInSandbox = runInSandbox;
exports.getProperties = getProperties;
exports.runAndGetAlerts = runAndGetAlerts;

var _vm = require('vm2');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _ramda = require('ramda');

var _compiler = require('./compiler');

var _db = require('./db');

var _i18n = require('./i18n');

var _module = require('./module');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Safely evaluates a piece of JS code as if it were running in the browser.
 *
 * @param {Object} sandbox
 * @param {String} code
 * @return {*}
 */
var runInBrowser = exports.runInBrowser = (0, _ramda.curry)(function (sandbox, code) {
    return new _vm.VM({ sandbox: sandbox }).run(code);
});

/**
 * Returns an instance of the Rung virtual machine
 *
 * @author Marcelo Haskell Camargo
 * @param {Object} translator - Map of strings to translate
 * @return {NodeVM}
 */
function createVM() {
    var strings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    var vm = new _vm.NodeVM({
        require: {
            external: true
        }
    });

    vm.freeze(_compiler.compileHTML, '__render__');
    vm.freeze((0, _i18n.translator)(strings), '_');

    return vm;
}

/**
 * Runs an app on a virtualized environment and returns its result as
 * native JS data
 *
 * @author Marcelo Haskell Camargo
 * @param {String} name - The unique identifier to track the app
 * @param {String} source - ES6 source to run
 * @param {Object} strings - Object containing the strings to translate
 * @param {String[][]} modules - Map of modules with [filename, source]
 * @return {Promise}
 */
function runInSandbox(name, source) {
    var strings = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var modules = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

    var evaluate = (0, _ramda.tryCatch)(function () {
        var vm = createVM(strings);

        // Pre-evaluate and inject in vm all necessary modules
        vm.options.require.mock = (0, _module.evaluateModules)(vm, modules);

        // Run with all modules! :)
        var result = vm.run(source, name + '.js');
        return (0, _bluebird.resolve)((0, _ramda.propOr)(result, 'default', result));
    }, _bluebird.reject);

    return evaluate();
}

/**
 * Tries to get the parameter types by running the script to get config.params
 *
 * @author Marcelo Haskell Camargo
 * @param {Object} extension - Must contain name and source
 * @param {Object} strings - Object with strings to translate
 * @param {String[][]} modules - Object with modules name and source
 * @return {Promise}
 */
function getProperties(extension, strings, modules) {
    return runInSandbox(extension.name, extension.source, strings, modules).then((0, _ramda.propOr)({}, 'config'));
}

/**
 * Runs an app with a context (with parameters) and gets the cards.
 * The result may be a string, a nullable value, an array...
 *
 * @author Marcelo Haskell Camargo
 * @param {Object} extension - Object containing name and source
 * @param {Object} context - Context to pass to the main function
 * @param {Object} strings - Object with strings to translate
 * @param {String[][]} modules - Modules with [filename, source]
 * @return {Promise}
 */
function runAndGetAlerts(extension, context, strings, modules) {
    return runInSandbox(extension.name, extension.source, strings, modules).then(function (app) {
        var runExtension = function runExtension() {
            return new _bluebird2.default(function (resolve, reject) {
                if ((0, _ramda.type)(app.extension) !== 'Function') {
                    return reject(new TypeError('Expected default exported expression to be a function'));
                }

                // Async vs sync extension
                return app.extension.length > 1 ? app.extension.call(null, context, resolve) : resolve(app.extension.call(null, context));
            });
        };

        return runExtension();
    }).tap(function (result) {
        return (0, _db.upsert)(extension.name, result.db);
    });
}
//# sourceMappingURL=vm.js.map